# -*- coding: utf-8 -*-
"""
A series of Functions to connect to the emonPI via it's API and download data
into Pandas dataframes / saves as CSV etc.
Date: 20231018
"""


#=============================================================================
def feed_id_from_name(feed_name,feed_df):
    """
    Parameters
    ----------
    feed_name : string
        Name of the Feed in emon - has to be unique
        
    feed_df : dataframe
        Dataframe of the feed data 
        
    Returns
    -------
    Integer - the ID of the feed or -999 if not found
    """

    try:
        record=feed_df[feed_df['name']==feed_name]
        if (len(record)==1):
            found_id=record['id'].iloc[0]
        else:
            found_id=-999
    except:
        found_id=-999
    return(found_id)            

##============================================================================
def feed_interval_from_name(feed_name,feed_df):
    """
    Parameters
    ----------
    feed_name : string
        Name of the Feed in emon - has to be unique
        
    feed_df : dataframe
        Dataframe of the feed data 
        
    Returns
    -------
    Integer - the Sampling Interval of the Data in Seconds 
    """

    try:
        record=feed_df[feed_df['name']==feed_name]
        if (len(record)==1):
            found_id=record['interval'].iloc[0]
        else:
            found_id=-999
    except:
        found_id=-999
    return(found_id)            

##============================================================================
def datetime_from_epoc_ms(ts_epoc_ms:int):
    """
    Take a time in ms from the epoch and returns the datetime 
    https://stackoverflow.com/questions/41613849/python-convert-timestamps-with-specific-timezone-to-datetime-in-utc
    https://stackoverflow.com/questions/31977563/how-do-you-convert-a-datetime-timestamp-from-one-timezone-to-another-timezone
    """
    from datetime import datetime
    import pytz 
    tzl=pytz.timezone('Europe/London')
    tzu=pytz.timezone('UTC')
    ts_datetime=datetime.fromtimestamp(ts_epoc_ms/1e3,tzl)
    ts_utc=ts_datetime.astimezone(tzu)
    return(ts_utc)
##============================================================================
def epoch_ms_from_datetime(ts_datetime):

    """
    Parameters
    ----------
    ts_datetime : datetime
        Timestamp in DateTime format

    Returns
    -------
    Timestamp in Miliseconds since the Epoc
    
    """
    ts_epoc_ms=ts_datetime.timestamp()*1e3
    return(ts_epoc_ms)
##============================================================================
def get_all_feed_data_dataframe(base_url,api_key):    
    import pandas as pd
    import requests
    """
    Parameters
    ----------
    base_url : string
        The base URL to the emon API typically http://<IP>/feed/

    api_key : string
        API key to access the data on emon  

    Returns
    -------
    A dataframe of all the Feed Meta Data
            
    """

    feed_list_url='list.json' #suffix of URL for list of feeds as JSON

    #Get feed data 
    call_url=base_url+feed_list_url+"&meta=1&apikey="+api_key
    #the meta=1 is required for all the fields including interval
    #print(call_url)
    response = requests.get(call_url)
    feed_data=response.json()
    feed_data_df=pd.DataFrame(feed_data)
    return(feed_data_df)    
##============================================================================
def dataframe_from_feed_single(start_time,end_time,feed_name, base_url,api_key):
    import requests    
    import pandas as pd
    
    """
    
    Parameters
    ----------
    start_time : datetime
        Start time of the Data Request in datetime format
    end_time : datetime
        End Time of the Data Request in datetime format
    feed_name : string
        Name of the feed as string - this needs to be unique 
    base_url : string
        The base URL to the emon API typically http://<IP>/feed/
    api_key : string
        API key to access the data on emon    

    Returns
    -------
    Returns a dataframe of Emon feed between the start and end time column
    names are "date_time" (index) and the feed name for the property

    """
    
    feed_df=get_all_feed_data_dataframe(base_url,api_key)
    
    
    #find feedID and feed interval
    feed_id_str=str(feed_id_from_name(feed_name, feed_df))
    
    feed_interval_str=str(feed_interval_from_name(feed_name,feed_df))
    #print('Feed Interval:',feed_interval_str)
    #get feed ID number from name.
    if (feed_id_str =='-999'):
        print('Sorry Feed name not found!')
        return()
    
    #generate the start and end
    start_ms_str = str(epoch_ms_from_datetime(start_time))
    end_ms_str   = str(epoch_ms_from_datetime(end_time))
    
    #generate feed query URL
    data_url=base_url+'data.json?id='+feed_id_str+'&start='+str(start_ms_str)
    #data_url+='&end='+str(end_ms_str)+'&interval='+feed_interval_str+'&apikey='+api_key
    data_url+='&end='+str(end_ms_str)+'&interval='+feed_interval_str+'&apikey='+api_key
    print(data_url)

    #make query and convert resultign data to dataframe
    data_response_json=requests.get(data_url).json() #get from API
    data_feed_df=pd.DataFrame(data_response_json,columns=['date_time' , feed_name]) #convert to dataframe
    data_feed_df['date_time']=data_feed_df['date_time'].apply(datetime_from_epoc_ms) #convert to datetime
    data_feed_df.set_index('date_time',inplace=True)
    data_feed_df.astype('Float32')
    
    
    return(data_feed_df)
#============================================================================
